import typing as t

import pygame as pg
from pygame.event import Event

__all__ = ("Keypad",)


class Keypad:
    """
    An object represeting a 16-key keypad of a standard keypad.
    """

    __slots__ = ("state",)

    def __init__(self) -> None:
        """
        Keypad Constructor.

        Attributes:
            state: State storing KEYUP and KEYDOWN status of the 16 keys.
        """
        self.state: t.List[int] = [0] * 16

    @property
    def keymap(self) -> t.Mapping[int, int]:
        """
        Mapping of pygame keys with locations on the keypad.
        """
        return {
            pg.K_1: 1,
            pg.K_2: 2,
            pg.K_3: 3,
            pg.K_4: 12,
            pg.K_q: 4,
            pg.K_w: 5,
            pg.K_e: 6,
            pg.K_r: 13,
            pg.K_a: 7,
            pg.K_s: 8,
            pg.K_d: 9,
            pg.K_f: 14,
            pg.K_z: 10,
            pg.K_x: 0,
            pg.K_c: 11,
            pg.K_v: 15,
        }

    def set(self, index: int) -> None:
        """
        Method to set a key in state array.

        Args:
            index: The key to set.
        """
        self.state[index] = 1

    def unset(self, index: int) -> None:
        """
        Method to unset a key in state array.

        Args:
            index: The key to unset.
        """
        self.state[index] = 0

    def handle(self, event: Event) -> None:
        """
        Handler for KEYDOWN.

        Args:
            event: A pygame Event.
        """
        if event.key in self.keymap:
            key = self.keymap[event.key]
            self.set(key)
            return key



import typing as t
from importlib.resources import files

import pygame

from .constants import COLORS, COLUMNS, ROWS

pygame.init()
icon = pygame.image.load(files("rechip8.static").joinpath("invader.png").open("rb"))

__all__ = ("Display",)


class Display:
    """
    Object for handling creation, rendering and deletion of the emulator
    window.
    """

    __slots__ = ("buffer", "multiplier", "screen")

    def __init__(self, screen: pygame.Surface, multiplier: int):
        """
        Display Constructor.

        Args:
            screen: A [pygame.Surface](https://www.pygame.org/docs/ref/surface.html#pygame.Surface) on which sprites are drawn.
            multiplier: Constant for scaling the window.

        Attributes:
            buffer (bytearray): Display Buffer; used to store sprite location on the window.
        """
        self.screen = screen
        self.multiplier = multiplier
        self.buffer: bytearray = bytearray(ROWS * COLUMNS)

    @classmethod
    def create(cls, multiplier: int) -> t.Self:
        """
        Create the window and sets caption and icon.

        Args:
            multiplier: Constant for scaling the window.
        """
        screen = pygame.display.set_mode(
            (COLUMNS * multiplier, ROWS * multiplier), vsync=True
        )
        pygame.display.set_caption("re:Chip8")
        pygame.display.set_icon(icon)
        self = cls(screen, multiplier)

        return self

    def refresh(self) -> None:
        """
        Refresh the display using
        [pygame.display.flip](https://www.pygame.org/docs/ref/display.html?highlight=pygame%20display%20flip#pygame.display.flip)
        """
        pygame.display.flip()

    def destroy(self) -> None:
        """
        Destroy the window and exit the emulator.
        """
        pygame.quit()
        raise SystemExit

    def wrap(self, x: int, y: int) -> int:
        """
        Wrap overflown/underflown pixels back at the opposite end.

        Args:
            x: The x position.
            y: The y position.

        Returns:
            loc: location of the cordinates in the display buffer.
        """
        x %= COLUMNS
        y %= ROWS

        loc = x + (y * COLUMNS)
        return loc

    def render(self) -> None:
        """
        Render the ON/OFF pixels onto the screen.
        """
        self.screen.fill(COLORS["OFF"])
        for i in range(ROWS * COLUMNS):
            x = (i % COLUMNS) * self.multiplier
            y = (i // COLUMNS) * self.multiplier

            if self.buffer[i]:
                pygame.draw.rect(
                    self.screen,
                    COLORS["ON"],
                    pygame.Rect(x, y, self.multiplier, self.multiplier),
                )
        self.refresh()

    def clear(self) -> None:
        """
        Clear the display buffer.
        """
        self.buffer = bytearray(ROWS * COLUMNS)



__all__ = ("Memory",)


class Memory:
    """
    Primary Memory of the CPU.
    """

    __slots__ = ("space",)

    def __init__(self) -> None:
        """
        Memory Constructor.

        Attributes:
            space (bytearray): A bytearray of size 4096 virtually representing CHP-8 memory.
        """
        self.space: bytearray = bytearray(4096)

    def load_binary(self, binary: bytes, offset: int = 0) -> None:
        """
        Load bytes onto the RAM.

        Arguments:
            binary: a bytes object.
            offset: From where to start loading the elements of the binary.
        """
        for i, data in enumerate(binary):
            self.space[i + offset] = data



import typing as t

__all__ = ("Opcode",)


class Opcode:
    """
    Operation code consisting of the information of the operation.
    """

    __slots__ = ("inst",)

    def __new__(cls, inst: int) -> t.Self:
        if isinstance(inst, int) and len(str(inst)) < 6:
            self = super(Opcode, cls).__new__(cls)
            return self

        else:
            raise ValueError(f"BAD OPCODE: {inst}")

    def __init__(self, inst: int) -> None:
        """
        Opcode Constructor.

        args:
            inst: instruction in the format of `hex(type|x|y|n|nn|nnn)`
        """
        self.inst = inst

    @property
    def type(self) -> int:
        """
        The type of operation. (first nibble)
        """
        value = self.inst & 0xF000
        return value

    @property
    def x(self) -> int:
        """
        General Purpose Register lookup. (second nibble)
        """
        value = (self.inst & 0x0F00) >> 8
        return value

    @property
    def y(self) -> int:
        """
        general purpose register lookup. (third nibble)
        """
        value = (self.inst & 0x00F0) >> 4
        return value

    @property
    def n(self) -> int:
        """
        4-bit numeric value. (fourth nibble)
        """
        value = self.inst & 0x000F
        return value

    @property
    def kk(self) -> int:
        """
        8-bit immediate. (third and fourth nibble)
        """
        value = self.inst & 0x00FF
        return value

    @property
    def nnn(self) -> int:
        """
        12-bit immediate memory address. (second, third and fourth nibble)
        """
        value = self.inst & 0x0FFF
        return value

    def __repr__(self) -> str:
        return f"Opcode(inst={hex(self.inst)})"



import logging
import typing as t
from importlib.resources import files
from random import randint

import pygame

from .constants import COLUMNS, CROSS, INIT_LOC_CONSTANT, ROWS
from .display import Display
from .keypad import Keypad
from .memory import Memory
from .opcode import Opcode

pygame.mixer.init()

__all__ = ("CPU",)


class CPU:
    """
    CPU of the Emulator.
    """

    __slots__ = (
        "DT",
        "I",
        "PC",
        "ST",
        "V",
        "display",
        "halt",
        "keypad",
        "memory",
        "op",
        "sound",
        "stack",
        "sync",
    )

    def __init__(self, display: Display, memory: Memory, keypad: Keypad) -> None:
        """
        CPU construtor.

        Args:
            memory: Primary Memory of size 4096 bytes.
            display: Display Handler for rendering sprites.
            keypad: 16-key hexadecimal keypad for taking input.

        Attributes:
            sound (pygame.mixer.Sound): A [pygame.mixer.Sound](https://www.pygame.org/docs/ref/mixer.html#pygame.mixer.Sound) object.
            op (Opcode): Opcode for identifying operation.

            V (List[int]): 16 General Purpose 8-bit registers.
            I (int): 16-bit Index register to store memory locations.
            DT (int): 8-bit Delay Timer.
            ST (int): 8-bit Sound Timer.
            PC (int): 16-bit register to store the currently executing address.
            stack (List[int]): Array of 16 16-bit values for nesting subroutines.

            halt (bool): Flag to check if the CPU is halted.
            sync (bool): Flag to sync the display with the timer.
        """
        # devices
        self.display = display
        self.memory = memory
        self.keypad = keypad
        self.sound: pygame.mixer.Sound = pygame.mixer.Sound(
            files("rechip8.static").joinpath("beep.mp3").open("rb")
        )
        self.op: Opcode = Opcode(inst=0x0000)

        # registers
        self.V: t.List[int] = [0] * 16
        self.I: int = 0
        self.DT: int = 0
        self.ST: int = 0
        self.PC: int = INIT_LOC_CONSTANT
        self.stack: t.List[int] = [0] * 16

        # flags
        self.halt: bool = False
        self.sync: bool = False

    def SYS_addr(self) -> None:
        """
        $0nnn - Jump to a machine code routine at nnn. (Edit: use kk instead)
        """
        optab = {0xE0: self.CLS, 0xEE: self.RET}
        optab[self.op.kk]()

    def CLS(self) -> None:
        """
        $00E0 - Clear the display.
        """
        self.display.clear()

    def RET(self) -> None:
        """
        $00EE - Return from a subroutine.
        """
        self.PC = self.stack.pop()

    def JP_addr(self) -> None:
        """
        $1nnn - Jump to location nnn. (PC = nnn)
        """
        self.PC = self.op.nnn

    def CALL_addr(self) -> None:
        """
        $2nnn - Call subroutine at nnn.
        """
        self.stack.append(self.PC)
        self.PC = self.op.nnn

    def SE_Vx_byte(self) -> None:
        """
        $3xkk - Skip next instruction if Vx = kk.
        """
        if self.V[self.op.x] == self.op.kk:
            self.PC += 2

    def SNE_Vx_byte(self) -> None:
        """
        $4xkk - Skip next instruction if Vx != kk.
        """
        if self.V[self.op.x] != self.op.kk:
            self.PC += 2

    def SE_Vx_Vy(self) -> None:
        """
        $5xy0 - Skip next instruction if Vx = Vy.
        """
        if self.V[self.op.x] == self.V[self.op.y]:
            self.PC += 2

    def LD_Vx_byte(self) -> None:
        """
        $6xkk - Set Vx = kk.
        """
        self.V[self.op.x] = self.op.kk

    def ADD_Vx_byte(self) -> None:
        """
        $7xkk - Set Vx = Vx + kk.
        """
        self.V[self.op.x] = (self.V[self.op.x] + self.op.kk) & 0xFF

    def JP_addr_8(self) -> None:
        """
        $8xyn -  Jump to a machine code routine at n.
        """
        optab = {
            0x0: self.LD_Vx_Vy,
            0x1: self.OR_Vx_Vy,
            0x2: self.AND_Vx_Vy,
            0x3: self.XOR_Vx_Vy,
            0x4: self.ADD_Vx_Vy,
            0x5: self.SUB_Vx_Vy,
            0x6: self.SHR_Vx_Vy,
            0x7: self.SUBN_Vx_Vy,
            0xE: self.SHL_Vx_Vy,
        }
        optab[self.op.n]()

    def LD_Vx_Vy(self) -> None:
        """
        $8xy0 - Set Vx = Vy.
        """
        self.V[self.op.x] = (self.V[self.op.y]) & 0xFF

    def OR_Vx_Vy(self) -> None:
        """
        $8xy1 - Set Vx = Vx OR Vy.
        """
        self.V[self.op.x] = (self.V[self.op.x] | self.V[self.op.y]) & 0xFF
        self.V[0xF] = 0

    def AND_Vx_Vy(self) -> None:
        """
        $8xy2 - Set Vx = Vx AND Vy.
        """
        self.V[self.op.x] = (self.V[self.op.x] & self.V[self.op.y]) & 0xFF
        self.V[0xF] = 0

    def XOR_Vx_Vy(self) -> None:
        """
        $8xy3 - Set Vx = Vx XOR Vy.
        """
        self.V[self.op.x] = (self.V[self.op.x] ^ self.V[self.op.y]) & 0xFF
        self.V[0xF] = 0

    def ADD_Vx_Vy(self) -> None:
        """
        $8xy4 - Set Vx = Vx + Vy, set VF = carry.
        """
        val = self.V[self.op.x] + self.V[self.op.y]
        self.V[self.op.x] = val & 0xFF

        if val > 0xFF:
            self.V[0xF] = 1
        else:
            self.V[0xF] = 0

    def SUB_Vx_Vy(self) -> None:
        """
        $8xy5 - Set Vx = Vx - Vy, set VF = NOT borrow.
        """
        x = self.V[self.op.x]
        y = self.V[self.op.y]

        self.V[self.op.x] = (x - y) & 0xFF

        if x < y:
            self.V[0xF] = 0
        else:
            self.V[0xF] = 1

    def SHR_Vx_Vy(self) -> None:
        """
        $8xy6 - Set Vx = Vx SHR 1.
        """
        prev = self.V[self.op.x]
        self.V[self.op.x] = self.V[self.op.y]
        self.V[self.op.x] = (self.V[self.op.x] >> 1) & 0xFF
        self.V[0xF] = prev & 1

    def SUBN_Vx_Vy(self) -> None:
        """
        $8xy7 - Set Vx = Vy - Vx, set VF = NOT borrow.
        """
        self.V[self.op.x] = (self.V[self.op.y] - self.V[self.op.x]) & 0xFF
        if self.V[self.op.x] > self.V[self.op.y]:
            self.V[0xF] = 0
        else:
            self.V[0xF] = 1

    def SHL_Vx_Vy(self) -> None:
        """
        $8xyE - Set Vx = Vx SHL 1.
        """
        y = self.V[self.op.y]

        self.V[self.op.x] = (y << 1) & 0xFF
        self.V[0xF] = (y & 0xFF) >> 7

    def SNE_Vx_Vy(self) -> None:
        """
        $9xy0 - Skip next instruction if Vx != Vy.
        """
        if self.V[self.op.x] != self.V[self.op.y]:
            self.PC += 2

    def LD_I_addr(self) -> None:
        """
        $Annn - Set I = nnn.
        """
        self.I = self.op.nnn

    def JP_V0_addr(self) -> None:
        """
        $Bnnn - Jump to location nnn + V0.
        """
        self.PC = self.op.nnn + self.V[0]

    def RND_Vx_byte(self) -> None:
        """
        $Cxkk - Set Vx = random byte AND kk.
        """
        self.V[self.op.x] = randint(0, 0xFF) & self.op.kk

    def DRW_Vx_Vy_nibble(self) -> None:
        """
        $Dxyn - Display n-byte sprite starting at memory location I at (Vx, Vy), set VF = collision. :)
        """
        if not self.sync:
            self.PC -= 2
            return

        x = self.V[self.op.x] % COLUMNS
        y = self.V[self.op.y] % ROWS

        self.V[0xF] = 0

        for i in range(self.op.n):
            sprite = self.memory.space[self.I + i]
            if (y + i) <= ROWS:
                for j in range(8):
                    if (x + j) <= COLUMNS:
                        px = (sprite >> (7 - j)) & 1
                        index = self.display.wrap(x + j, y + i)

                        if px == 1 and self.display.buffer[index] == 1:
                            self.V[0xF] = 1

                        self.display.buffer[index] ^= px

        self.sync = False

    def Jp_addr_E(self) -> None:
        """
        $Exkk - Jump to a machine code routine at kk.
        """
        optab = {0x9E: self.SKP_Vx, 0xA1: self.SKNP_Vx}
        optab[self.op.kk]()

    def SKP_Vx(self) -> None:
        """
        $Ex9E - Skip next instruction if key with the value of Vx is pressed.
        """
        if self.keypad.state[self.V[self.op.x] & 0xF]:
            self.PC += 2

    def SKNP_Vx(self) -> None:
        """
        $ExA1 - Skip next instruction if key with the value of Vx is not pressed.
        """
        if not self.keypad.state[self.V[self.op.x] & 0xF]:
            self.PC += 2

    def Jp_addr_F(self) -> None:
        """
        $Fxkk - Jump to a machine code routine at kk.
        """
        optab = {
            0x07: self.LD_Vx_DT,
            0x0A: self.LD_Vx_K,
            0x15: self.LD_DT_Vx,
            0x18: self.LD_ST_Vx,
            0x1E: self.ADD_I_Vx,
            0x29: self.LD_F_Vx,
            0x33: self.LD_B_Vx,
            0x55: self.LD_I_Vx,
            0x65: self.LD_Vx_I,
        }
        optab[self.op.kk]()

    def LD_Vx_DT(self) -> None:
        """
        $Fx07 - Set Vx = delay timer value.
        """
        self.V[self.op.x] = self.DT

    def LD_Vx_K(self) -> None:
        """
        $Fx0A - Wait for a key press, store the value of the key in Vx.
        """
        self.halt = True

    def LD_DT_Vx(self) -> None:
        """
        $Fx15 - Set delay timer = Vx.
        """
        self.DT = self.V[self.op.x]

    def LD_ST_Vx(self) -> None:
        """
        $Fx18 - Set sound timer = Vx.
        """
        self.ST = self.V[self.op.x]

    def ADD_I_Vx(self) -> None:
        """
        $Fx1E - Set I = I + Vx.
        """
        self.I += self.V[self.op.x]

    def LD_F_Vx(self) -> None:
        """
        $Fx29 - Set I = location of sprite for digit Vx.
        """
        self.I = (self.V[self.op.x] * 5) & 0x0FFF

    def LD_B_Vx(self) -> None:
        """
        $Fx33 - Store BCD representation of Vx in memory locations I, I+1, and I+2.
        """
        self.memory.space[self.I] = self.V[self.op.x] // 100
        self.memory.space[self.I + 1] = (self.V[self.op.x] % 100) // 10
        self.memory.space[self.I + 2] = self.V[self.op.x] % 10

    def LD_I_Vx(self) -> None:
        """
        $Fx55 - Store registers V0 through Vx in memory starting at location I.
        """
        for i in range(self.op.x + 1):
            self.memory.space[self.I + i] = self.V[i]
        self.I += self.op.x + 1

    def LD_Vx_I(self) -> None:
        """
        $Fx65 - Read registers V0 through Vx from memory starting at location I.
        """
        for i in range(self.op.x + 1):
            self.V[i] = self.memory.space[self.I + i]
        self.I += self.op.x + 1

    @property
    def optable(self) -> t.Mapping[int, t.Callable[..., None]]:
        """
        Opcode function lookup table.

        Returns:
            dict: opcode.type:subroutine mapping.
        """
        return {
            0x0: self.SYS_addr,
            0x1: self.JP_addr,
            0x2: self.CALL_addr,
            0x3: self.SE_Vx_byte,
            0x4: self.SNE_Vx_byte,
            0x5: self.SE_Vx_Vy,
            0x6: self.LD_Vx_byte,
            0x7: self.ADD_Vx_byte,
            0x8: self.JP_addr_8,
            0x9: self.SNE_Vx_Vy,
            0xA: self.LD_I_addr,
            0xB: self.JP_V0_addr,
            0xC: self.RND_Vx_byte,
            0xD: self.DRW_Vx_Vy_nibble,
            0xE: self.Jp_addr_E,
            0xF: self.Jp_addr_F,
        }

    def beep(self) -> None:
        """
        Make a beep sound.
        """
        self.sound.play()

    def cycle(self) -> None:
        """
        Fetch, Decode and Execute instructions from the memory.
        """
        fetch = (self.memory.space[self.PC] << 8) | self.memory.space[self.PC + 1]
        self.op = Opcode(fetch)
        self.PC += 2

        try:
            self.optable[(self.op.type & 0xF000) >> 12]()
        except KeyError:
            logging.error(f"{CROSS} Opcode not found: {hex(self.op.inst)}")
            self.display.destroy()

    def handle_timers(self) -> None:
        """
        Decrement ST and DT.
        """
        if self.DT > 0:
            self.DT -= 1

        if self.ST > 0:
            self.ST = 0
            self.beep()



import argparse
import logging
from importlib.resources import files

import pygame

from .core import (CPU, INIT_LOC_CONSTANT, TICK, WAVE, Display, Keypad,
                         Memory)

logging.basicConfig(
    format="%(asctime)s:%(msecs)03d (%(levelname)s/%(module)s): %(message)s",
    level=logging.DEBUG,
    encoding="utf-8",
    datefmt="%M:%S",
)


class reChip8:
    """
    As the main entry point for re:Chip8 emulator,
    this class takes care of interfacing with the user and
    the internal devices.
    """

    __slots__ = ("cpu", "display", "keypad", "memory")

    def __init__(self, rom: str, mul: int) -> None:
        """
        reChip8 Constructor.
        The constructor is responsible for loading font and rom, and also initializing other
        devices.

        Args:
            rom: Path to the ROM file.
            mul: The screen size multiplier.

        Attributes:
            memory (Memory): Primary Memory of size 4096 bytes.
            display (Display): Display Handler for rendering sprites.
            keypad (Keypad): 16-key hexadecimal keypad for taking input.
            cpu (CPU): Object representing Central Processing Unit of the emulator.
        """
        self.memory: Memory = Memory()
        self.load_font()
        self.load_rom(rom)
        self.display: Display = Display.create(multiplier=mul)
        self.keypad: Keypad = Keypad()
        self.cpu: CPU = CPU(
            display=self.display, memory=self.memory, keypad=self.keypad
        )

    def load_font(self) -> None:
        """
        Load Font from the `/bin/FONT` file in memory from location `0x0`
        """
        self.memory.load_binary(files("rechip8.bin").joinpath("FONT").read_bytes())
        logging.info(f"{TICK} Successfully loaded Fontset at location 0x0")

    def load_rom(self, rom: str) -> None:
        """
        Load ROM in memory from location `0x200` (512)

        Args:
            rom: Path to the ROM file.
        """
        self.memory.load_binary(open(rom, "rb").read(), offset=INIT_LOC_CONSTANT)
        logging.info(
            f"{TICK} Successfully loaded ROM at location {hex(INIT_LOC_CONSTANT)}"
        )

    def tick(self) -> None:
        """
        Method representing a single tick from the emulator.
        """
        for ic in range(10):
            if not self.cpu.halt:
                self.cpu.cycle()
                self.cpu.sync = not ic
            self.display.render()

        self.cpu.handle_timers()

    def run(self) -> None:
        """
        Step through the emulation indefinitely.
        """
        is_running = True
        while is_running:
            self.tick()

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    is_running = False
                if event.type == pygame.KEYDOWN:
                    self.keypad.handle(event)
                if event.type == pygame.KEYUP:
                    if event.key in self.keypad.keymap:
                        key = self.cpu.keypad.keymap[event.key]
                        if self.cpu.halt:
                            self.cpu.V[self.cpu.op.x] = key
                            self.cpu.halt = False
                        self.keypad.unset(key)

        self.display.destroy()


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        prog="re:Chip8", description="Chip-8 Virtual Machine."
    )
    parser.add_argument("rom", help="Path to the rom file.")
    parser.add_argument(
        "-S",
        "--scale",
        help="Scale up or down the Window size.",
        type=int,
        default=10,
    )
    args = parser.parse_args()

    rechip8 = reChip8(args.rom, args.scale)

    try:
        rechip8.run()
    except KeyboardInterrupt:
        logging.info(f"bye {WAVE}")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rechip8', 'rechip8.core']

package_data = \
{'': ['*'], 'rechip8': ['bin/*', 'static/*']}

install_requires = \
['pygame>=2.2.0,<3.0.0']

setup_kwargs = {
    'name': 'rechip8',
    'version': '1.4.0',
    'description': 'A CHIP-8 Emulator written in Python.',
    'long_description': '<p align="center"> <img src="https://raw.githubusercontent.com/mooncell07/reChip8/master/docs/img/reChip8.png" height=100> </p>\n\n<h1 align="center"> <a href="https://mooncell07.github.io/reChip8/">reChip8</a> </h1>\n<h2 align="center">\n<img alt="Stargazers" src="https://img.shields.io/github/stars/mooncell07/reChip8?style=for-the-badge&logo=starship&color=C9CBFF&logoColor=D9E0EE&labelColor=302D41">\n<img alt="Issues" src="https://img.shields.io/github/issues/mooncell07/reChip8?style=for-the-badge&logo=gitbook&color=B5E8E0&logoColor=D9E0EE&labelColor=302D41">\n<img alt="Releases" src="https://img.shields.io/github/license/mooncell07/reChip8?style=for-the-badge&logo=github&color=F2CDCD&logoColor=D9E0EE&labelColor=302D41"/>\n</h2>\n\nre:Chip8 (Formerly: Lemon.pie) is a **[CHIP-8](https://en.wikipedia.org/wiki/CHIP-8)** Emulator written in Python using Pygame.\nThis project aims at implementing the classic variant of CHIP-8 Virtual Machine. Currently a Work in Progress.\n\n## Demo\n\n![CHIP-8 Logo](https://raw.githubusercontent.com/mooncell07/reChip8/master/docs/img/chip8-logo.png)\n\n![PONG](https://raw.githubusercontent.com/mooncell07/reChip8/master/docs/img/gameplay-pong.png)\n\n![Animal Race](https://raw.githubusercontent.com/mooncell07/reChip8/master/docs/img/gameplay-animal-race.png)\n\nUsage and more info is available on [reChip8](https://mooncell07.github.io/reChip8/).\n',
    'author': 'mooncell07',
    'author_email': '80042274+mooncell07@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)
